
clear; clc; close all;

% Define the function and its derivative
f   = @(x) x.^2 - 2;
df  = @(x) 2*x;
true_root = sqrt(2);

% Tolerance
tol = 1e-6;

%% Bisection Method
a = 0; b = 2;
[sol_bisec, traj_bisec] = bisec_rk(f, a, b, tol);
iter_bisec = 1:length(traj_bisec);

%% Secant Method
x0 = 0; x1 = 2;
[sol_secant, traj_secant] = secant_rk(f, x0, x1, tol);
iter_secant = 1:length(traj_secant);

%% Newton's Method
x0_newton = 1; % initial guess near the root
[sol_newton, traj_newton] = newton_rk(f, df, x0_newton, tol);
iter_newton = 1:length(traj_newton);

%% Plot the error (|x_n - true_root|) on a semilog scale
error_bisec   = abs(traj_bisec - true_root);
error_secant  = abs(traj_secant - true_root);
error_newton  = abs(traj_newton - true_root);


figure;
semilogy(iter_bisec, error_bisec, '-o', 'DisplayName', 'Bisection');
hold on; 
semilogy(iter_secant, error_secant, '-s', 'DisplayName', 'Secant');
semilogy(iter_newton, error_newton, '-^', 'DisplayName', 'Newton');
grid on;
xlabel('Iteration Number');
ylabel('Absolute Error |x_n - \surd2|');
title('Convergence Speed Comparison (Semilog Plot)');
legend('Location','best');
hold off;

%% plot order of convergence
figure;
% Bisection method: Expect linear convergence
loglog(error_bisec(1:end-1), error_bisec(2:end), '-o', 'DisplayName', 'Bisection');
hold on; grid on;
% Secant method: Expect superlinear convergence (~1.618 order)
loglog(error_secant(1:end-1), error_secant(2:end), '-s', 'DisplayName', 'Secant');
% Newton's method: Expect quadratic convergence
loglog(error_newton(1:end-1), error_newton(2:end), '-^', 'DisplayName', 'Newton');


% Reference Lines
ref_x = logspace(log10(min(error_secant(1:end-1))), ...
                 log10(max(error_secant(1:end-1))), 100);

loglog(ref_x, ref_x, '--', 'DisplayName', 'Linear (y = x)'); % Linear convergence
loglog(ref_x, ref_x.^1.618, '--', 'DisplayName', 'Superlinear (y = x^{1.618})'); % Secant
loglog(ref_x, ref_x.^2, '--', 'DisplayName', 'Quadratic (y = x^2)'); % Newton

xlabel('Error at iteration n: |x_n - \surd2|');
ylabel('Error at iteration n+1: |x_{n+1} - \surd2|');
title('Convergence Order Comparison (Log-Log Plot)');
legend('Location', 'best');
hold off;