function [sol, traj] = secant_rk(func, x0, x1, tol)
    traj = [x0, x1];  % record the initial guesses

    while abs(x1 - x0) > tol
        f0 = func(x0);
        f1 = func(x1);
        % Avoid division by zero
        if f1 - f0 == 0
            error('Division by zero encountered in the secant method.');
        end
        x_new = x1 - f1*(x1 - x0)/(f1 - f0);
        traj(end+1) = x_new; 
        x0 = x1;
        x1 = x_new;
    end

    sol = x1;
end